﻿using System;
using System.Drawing;
using System.Windows.Forms;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;

using AmbilightControlTest.Core;

namespace AmbilightControlTest.Ui.Controls
{
    public class AmbiFrameControl : Control
    {

        #region constructor
        public AmbiFrameControl()
        {

            SetStyle( ControlStyles.SupportsTransparentBackColor |
                      ControlStyles.OptimizedDoubleBuffer |
                      ControlStyles.UserPaint |
                      ControlStyles.AllPaintingInWmPaint// |
                      //ControlStyles.ResizeRedraw
                      , true );

            BackColor = Color.Transparent;

            Size = new Size( 150, 50 );

            gradientColors = new ObservableCollection<Color>();
            gradientColors.CollectionChanged += GradientColors_CollectionChanged;

            gradientOrientation = Orientation.Horizontal;
            transparencySide = TransparencySides.None;

            isBitmapCacheDirty = true;

        }
        #endregion

        #region enums

        public enum TransparencySides
        {
            None,
            Top,
            Bottom,
            Left,
            Right
        }

        #endregion

        #region properties

        private Orientation gradientOrientation;

        [Category("Ambilight")]
        [Description("The orientation of the gradient.")]
        public Orientation GradientOrientation
        {
            get { return gradientOrientation; }
            set
            {
                gradientOrientation = value;
                ReDraw();
            }
        }

        private ObservableCollection<Color> gradientColors;
        [DesignerSerializationVisibility( DesignerSerializationVisibility.Content )]
        [Category( "Ambilight" )]
        [Description( "The color list for the gradient." )]
        public ObservableCollection<Color> GradientColors
        {
            get { return gradientColors; }
        }


        private TransparencySides transparencySide;
        [Category( "Ambilight" )]
        [Description( "The side where the color is faded to transparent." )]
        public TransparencySides TransparencySide
        {
            get { return transparencySide; }
            set
            {
                transparencySide = value;
                ReDraw();
            }
        }

        #endregion

        #region variables

        private Bitmap cacheBitmap;
        private bool isBitmapCacheDirty;

        #endregion

        #region methods

        private void ReDraw()
        {
            isBitmapCacheDirty = true;
            Invalidate();
        }

        private void RenderBitmap()
        {

            Rectangle rect = new Rectangle( 0, 0, Width, Height );

            if (cacheBitmap != null)
                cacheBitmap.Dispose();

            cacheBitmap = new Bitmap( rect.Width, rect.Height, PixelFormat.Format32bppArgb );
           
            if (gradientColors.Count >= 2 && gradientColors[gradientColors.Count - 1] != Color.Empty)
            {
                #region gradient

                using (Graphics g = Graphics.FromImage( cacheBitmap ))
                {
                    g.Clear( Color.Transparent );

                    int angle = gradientOrientation == Orientation.Vertical ? 90 : 0;
                    LinearGradientBrush br = new LinearGradientBrush( rect, Color.Transparent, Color.Transparent, angle, false );
                    br.WrapMode = WrapMode.Tile;
                    ColorBlend cb = new ColorBlend();
                    float[] positions = new float[gradientColors.Count];
                    Color[] colors = new Color[gradientColors.Count];

                    for (int i = 0; i < gradientColors.Count; i++)
                    {
                        colors[i] = gradientColors[i];
                        positions[i] = i / (float)( positions.Length - 1 );
                    }

                    cb.Positions = positions;
                    cb.Colors = colors;
                    br.InterpolationColors = cb;

                    g.FillRectangle( br, rect );
                }

                #endregion

                #region transparency

                LockBitmap lockedBmp = new LockBitmap( cacheBitmap );
                lockedBmp.LockBits();
                if (transparencySide == TransparencySides.Top)
                {
                    int start = lockedBmp.Height / 2;
                    for (int y = start; y >= 0; y--)
                    {
                        byte currentTransparency = (byte)Map( y, start, 0, 255, 0 );
                        for (int x = 0; x < lockedBmp.Width; x++)
                        {
                            Color pixel = lockedBmp.GetPixel( x, y );
                            pixel = Color.FromArgb( currentTransparency, pixel.R, pixel.G, pixel.B );
                            lockedBmp.SetPixel( x, y, pixel );
                        }
                    }
                }
                else if (transparencySide == TransparencySides.Bottom)
                {
                    int start = lockedBmp.Height / 2;
                    for (int y = start; y < lockedBmp.Height; y++)
                    {
                        byte currentTransparency = (byte)Map( y, start, lockedBmp.Height, 255, 0 );
                        for (int x = 0; x < lockedBmp.Width; x++)
                        {
                            Color pixel = lockedBmp.GetPixel( x, y );
                            pixel = Color.FromArgb( currentTransparency, pixel.R, pixel.G, pixel.B );
                            lockedBmp.SetPixel( x, y, pixel );
                        }
                    }
                }
                else if (transparencySide == TransparencySides.Left)
                {
                    int start = lockedBmp.Width / 2;
                    for (int x = start; x >= 0; x--)
                    {
                        byte currentTransparency = (byte)Map( x, 0, start, 0, 255 );
                        for (int y = 0; y < lockedBmp.Height; y++)
                        {
                            Color pixel = lockedBmp.GetPixel( x, y );
                            pixel = Color.FromArgb( currentTransparency, pixel.R, pixel.G, pixel.B );
                            lockedBmp.SetPixel( x, y, pixel );
                        }
                    }
                }
                else if (transparencySide == TransparencySides.Right)
                {
                    int start = lockedBmp.Width / 2;
                    for (int x = start; x < lockedBmp.Width; x++)
                    {
                        byte currentTransparency = (byte)Map( x, start, lockedBmp.Width, 255, 0 );
                        for (int y = 0; y < lockedBmp.Height; y++)
                        {
                            Color pixel = lockedBmp.GetPixel( x, y );
                            pixel = Color.FromArgb( currentTransparency, pixel.R, pixel.G, pixel.B );
                            lockedBmp.SetPixel( x, y, pixel );
                        }
                    }
                }
                lockedBmp.UnlockBits();
                #endregion

            }

            isBitmapCacheDirty = false;

        }

        #endregion

        #region functions

        /// <summary>
        /// Maps a value to another range
        /// </summary>
        /// <param name="value">The value which be mapped.</param>
        /// <param name="fromSource">The minimum value of the source value.</param>
        /// <param name="toSource">The maximum value of the source value.</param>
        /// <param name="fromTarget">The minimum value of the mapped target value.</param>
        /// <param name="toTarget">The maximum value of the mapped target value.</param>
        ///  <returns></returns> 
        private float Map(float value, float fromSource, float toSource, float fromTarget, float toTarget)
        {
            return ( value - fromSource ) / ( toSource - fromSource ) * ( toTarget - fromTarget ) + fromTarget;
        }

        #endregion

        #region callbacks

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint( e );

            if (isBitmapCacheDirty) RenderBitmap();

            if ( cacheBitmap != null)
                e.Graphics.DrawImageUnscaled( cacheBitmap, 0, 0 );
        }

        private void GradientColors_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (gradientColors.Count >= 2 && gradientColors[gradientColors.Count - 1] != Color.Empty)
            {
                ReDraw();
            }
        }

        protected override void OnSizeChanged(EventArgs e)
        {
            base.OnSizeChanged( e );
            ReDraw();
        }

        #endregion

    }
}
